/*   CS580 HW   */
#include    "stdafx.h"  
#include	"Gz.h"
#include	"disp.h"

unsigned short clampus(unsigned short *x, int min, int max){
	if(*x < min)
		*x = min;
	else if(*x > max)
		*x = max;
	return *x;
}

int GzNewFrameBuffer(char** framebuffer, int width, int height){
/* create a framebuffer:
 -- allocate memory for framebuffer : (sizeof)GzPixel x width x height
 -- pass back pointer 
*/
	*framebuffer = new char[3*sizeof(char)*width*height];
	if(framebuffer && width && height) 
		return GZ_SUCCESS;
	return GZ_FAILURE;
}

int GzNewDisplay(GzDisplay	**display, GzDisplayClass dispClass, int xRes, int yRes)
{

/* create a display:
  -- allocate memory for indicated class and resolution
  -- pass back pointer to GzDisplay object in display
*/
	if(dispClass != GZ_Z_BUFFER_RENDER)
		return GZ_FAILURE;
	
	if(!(*display = (GzDisplay *)malloc(sizeof(GzDisplay))))
		return GZ_FAILURE;

	GzDisplay empty = {NULL};
	memcpy(*display, &empty, sizeof(GzDisplay));
	
	(*display)->xres = xRes;
	clampus(&((*display)->xres), 1, MAXXRES);
	(*display)->yres = yRes;
	clampus(&((*display)->yres), 1, MAXYRES);

	(*display)->dispClass = dispClass;
	(*display)->fbuf = (GzPixel *)malloc(sizeof(GzPixel)*xRes*yRes);
	(*display)->open = 1;

	return GZ_SUCCESS;
}


int GzFreeDisplay(GzDisplay	*display)
{
/* clean up, free memory */
	if(display){
		if(display->fbuf){
			free(display->fbuf);
			display->fbuf = NULL;
		}
		free(display);
		display = NULL;
	}
	return GZ_SUCCESS;
}


int GzGetDisplayParams(GzDisplay *display, int *xRes, int *yRes, GzDisplayClass	*dispClass)
{
/* pass back values for an open display */
	if(display){
		*xRes = display->xres;
		*yRes = display->yres;
		*dispClass = display->dispClass;
		return GZ_SUCCESS;
	}
	return GZ_FAILURE;
}


int GzInitDisplay(GzDisplay	*display)
{
/* set everything to some default values - start a new frame */
	if(display){
		for(int i = 0; i < display->xres*display->yres; i++){
			display->fbuf[i].red = 2048;
			display->fbuf[i].green = 2048;
			display->fbuf[i].blue = 2048;
			display->fbuf[i].alpha = 4095;
			display->fbuf[i].z = (GzDepth)1600000000.0f;
		}
		return GZ_SUCCESS;
	}
	return GZ_FAILURE;
}


int GzPutDisplay(GzDisplay *display, int i, int j, GzIntensity r, GzIntensity g, GzIntensity b, GzIntensity a, GzDepth z)
{
/* write pixel values into the display */
	if(display){
		if(i > 0 && i < display->xres && j > 0 && j < display->yres){
			display->fbuf[ARRAY(i, j)].red = r;
			display->fbuf[ARRAY(i, j)].green = g;
			display->fbuf[ARRAY(i, j)].blue = b;
			display->fbuf[ARRAY(i, j)].alpha = a;
			display->fbuf[ARRAY(i, j)].z = z;
		}
		return GZ_SUCCESS;
	}
	return GZ_FAILURE;
}


int GzGetDisplay(GzDisplay *display, int i, int j, GzIntensity *r, GzIntensity *g, GzIntensity *b, GzIntensity *a, GzDepth *z)
{
	/* pass back pixel value in the display */
	/* check display class to see what vars are valid */
	if(display){
		if(i > 0 && i < display->xres && j > 0 && j < display->yres){
			*r = display->fbuf[ARRAY(i, j)].red;
			*g = display->fbuf[ARRAY(i, j)].green;
			*b = display->fbuf[ARRAY(i, j)].blue;
			*a = display->fbuf[ARRAY(i, j)].alpha;
			*z = display->fbuf[ARRAY(i, j)].z;
		}
		return GZ_SUCCESS;
	}
	return GZ_FAILURE;
}


int GzFlushDisplay2File(FILE* outfile, GzDisplay *display)
{
	/* write pixels to ppm file based on display class -- "P6 %d %d 255\r" */
	if(display){
		unsigned short red;
		unsigned short green;
		unsigned short blue;

		fprintf(outfile, "P6 %d %d 255\r", display->xres, display->yres);
		for(int i = 0; i < display->xres*display->yres; i++){
			red = display->fbuf[i].red;
			green = display->fbuf[i].green;
			blue = display->fbuf[i].blue;

			red = clampus(&red, 0, 4095) >> 4;
			green = clampus(&green, 0, 4095) >> 4;
			blue = clampus(&blue, 0, 4095) >> 4;
			fprintf(outfile, "%c%c%c", (char)red, (char)green, (char)blue);
		}
		return GZ_SUCCESS;
	}
	return GZ_FAILURE;
}

int GzFlushDisplay2FrameBuffer(char* framebuffer, GzDisplay *display)
{
	/* write pixels to framebuffer: 
		- Put the pixels into the frame buffer
		- Caution: store the pixel to the frame buffer as the order of blue, green, and red 
		- Not red, green, and blue !!!
	*/
	if(display && framebuffer){
		for(int i = 0; i < display->xres*display->yres; i++){
			unsigned short blue = display->fbuf[i].blue;
			unsigned short green = display->fbuf[i].green;
			unsigned short red = display->fbuf[i].red;

			blue = clampus(&blue, 0, 4095) >> 4;
			green = clampus(&green, 0, 4095) >> 4;
			red = clampus(&red, 0, 4095) >> 4;

			int index = 3*i;
			framebuffer[index++] = (char)blue;
			framebuffer[index++] = (char)green;
			framebuffer[index] = (char)red;
		}
		return GZ_SUCCESS;
	}
	return GZ_FAILURE;
}