/* CS580 Homework 3 */

#include	"stdafx.h"
#include	"stdio.h"
#include	"math.h"
#include	"Gz.h"
#include	"rend.h"
#include	"float.h"

#define		PI		3.141f
#define		Z_MAX	1600000000.0f

/* NOT part of API - just for general assistance */

short	ctoi(float color)		/* convert float color to GzIntensity short */
{
  return(short)((int)(color * ((1 << 12) - 1)));
}

void normalize(float coord[3]){
	float tot = .0f;
	for(int i = 0; i < 3; i++)
		tot += coord[i]*coord[i];
	tot = pow(tot, 0.5f);
	for(int i = 0; i < 3; i++)
		coord[i] /= tot;
}

void multMatCoord(GzMatrix matrix, GzCoord coord){
	float xformCoords[4];
	float sum;
		for(int i = 0; i < 4; i++){
			sum = .0f;
			for(int j = 0; j < 4; j++)
				sum += (j == 3) ? matrix[i][j] : coord[j]*matrix[i][j];
			xformCoords[i] = sum;
		}
		for(int i = 0; i < 3; i++)
			coord[i] = xformCoords[i]/xformCoords[3];
}

void multMat(GzMatrix* dest, GzMatrix right){
	GzMatrix product;
	for(int i = 0; i < 4; i++){
		for (int j = 0; j < 4; j++){
			float prod = .0f;
			for(int k = 0; k < 4; k++)
					prod += (*dest)[i][k]*right[k][j];
			product[i][j] = prod;
		}
	}
	memcpy(dest, product, sizeof(GzMatrix));	
}

//Lin. interpolation to find z, given a tri. and x, y
float interpolate(GzCoord verts[3], float x, float y){
	float a, b, c, d;
	float u[3], v[3];
	float dx[3], dy[3], dz[3], hx[3], hy[3];

	for(int i = 0; i < 3; i++){
		dx[i] = verts[(i+1)%3][0] - verts[i][0];
		dy[i] = verts[(i+1)%3][1] - verts[i][1];
		dz[i] = verts[(i+1)%3][2] - verts[i][2];
		hx[i] = verts[i][0];
		hy[i] = verts[i][1];
	}

	u[0] = dx[0];
	u[1] = dy[0];
	u[2] = dz[0];
	v[0] = -dx[2];
	v[1] = -dy[2];
	v[2] = -dz[2];
	a = (u[1]*v[2]) - (u[2]*v[1]);
	b = (u[2]*v[0]) - (u[0]*v[2]);
	c = (u[0]*v[1]) - (u[1]*v[0]);
	d = -(a*verts[0][0] + b*verts[0][1] + c*verts[0][2]);
	return -((a*x + b*y + d) / c);
}

/*
 * When swapping coords, this also swaps all properties of each coord
 * i is starting index, j is destination index
 */

void swapCoordList(GzCoord *coordList, GzCoord *normalList, GzTextureIndex *uvList, int i, int j){
	GzCoord temp;
	memcpy(&temp, coordList[i], sizeof(GzCoord));
	memcpy(coordList[i], coordList[j], sizeof(GzCoord));
	memcpy(coordList[j], &temp, sizeof(GzCoord));

	memcpy(&temp, normalList[i], sizeof(GzCoord));
	memcpy(normalList[i], normalList[j], sizeof(GzCoord));
	memcpy(normalList[j], &temp, sizeof(GzCoord));

	memcpy(&temp, uvList[i], sizeof(GzTextureIndex));
	memcpy(uvList[i], uvList[j], sizeof(GzTextureIndex));
	memcpy(uvList[j], &temp, sizeof(GzTextureIndex));
}

void computeColor(GzColor out, GzRender *render, GzCoord n){
	float ndotl, ndote, rdotes;

	normalize(n);
	for(int i = 0; i < 3; i++)
		out[i] = render->ambientlight.color[i] * render->Ka[i];

	for(int i = 0; i < render->numlights; i++){
		ndotl = n[0]*render->lights[i].direction[0] + n[1]*render->lights[i].direction[1] + n[2]*render->lights[i].direction[2];
		ndote = -n[2];

		if(ndotl*ndote > .0f){
			if(ndotl < .0f){
				for(int j = 0; j < 3; j++)
					n[j] = -n[j];
				ndotl = -ndotl;
			}
			GzCoord r = {2*ndotl*n[0], 2*ndotl*n[1], 2*ndotl*n[2]};
			for(int j = 0; j < 3; j++)
				r[j] -= render->lights[i].direction[j];
			rdotes = pow(-r[2], render->spec);

			for(int j = 0; j < 3; j++){
				out[j] += render->lights[i].color[j]*ndotl*render->Kd[j];
				out[j] += render->lights[i].color[j]*rdotes*render->Ks[j];
			}
		}
	}
}

void colorPixel(GzCoord coordList[3], GzCoord normalList[3], GzColor vertColors[3], GzRender *render, 
				float x, float y, float z){
	GzColor finalColor;
	float temp[3];
	GzCoord normal;

		switch(render->interp_mode){
			case GZ_NONE:
				for(int i = 0; i < 3; i++)
					finalColor[i] = render->flatcolor[i];
				break;

			case GZ_COLOR:
				for(int i = 0; i < 3; i++)
					temp[i] = coordList[i][2];
				for(int i = 0; i < 3; i++){
					for(int j = 0; j < 3; j++)
						coordList[j][2] = vertColors[j][i];
					finalColor[i] = interpolate(coordList, x, y);
				}
				for(int i = 0; i < 3; i++)
					coordList[i][2] = temp[i];
				break;

			case GZ_NORMALS:
				for(int i = 0; i < 3; i++)
					temp[i] = coordList[i][2];
				for(int i = 0; i < 3; i++){
					for(int j = 0; j < 3; j++)
						coordList[j][2] = normalList[j][i];
					normal[i] = interpolate(coordList, x, y);
				}
				for(int i = 0; i < 3; i++)
					coordList[i][2] = temp[i];
				normalize(normal);
				computeColor(finalColor, render, normal);
				break;

			default:
				break;
		}

		for(int i = 0; i < 3; i++){
			if(finalColor[i] < .0f)
				finalColor[i] = .0f;
			else if(finalColor[i] > 1.0f)
				finalColor[i] = 1.0f;
		}

		GzPutDisplay(render->display, (int)x, (int)y, 
					ctoi(finalColor[0]),
					ctoi(finalColor[1]),
					ctoi(finalColor[2]),
					4095, (GzDepth)z);
}

bool lee(GzCoord coordList[3], GzRender *render, float x, float y){
	float dx[3], dy[3], hx[3], hy[3];
	bool leePass[3];
	for(int j = 0; j < 3; j++){
		dx[j] = coordList[(j+1)%3][0] - coordList[j][0];
		dy[j] = coordList[(j+1)%3][1] - coordList[j][1];
		hx[j] = coordList[j][0];
		hy[j] = coordList[j][1];
	}

	for(int j = 0; j < 3; j++){
		float leeCheck = (dy[j]*(x - hx[j])) - (dx[j]*(y - hy[j]));
		leePass[j] = (leeCheck > .0f) ? true : false;
		if(leeCheck == .0f){
			switch(j){
				case 0:
					leePass[j] = true;
					break;
				case 1:
				case 2:
					leePass[j] = (dy[j]/dx[j] > 0) ? true : false;
					break;
				default:
					break;
			}
		}
	}

	return(leePass[0] && leePass[1] && leePass[2] ||
		!(leePass[0] || leePass[1] || leePass[2]));
}



void sortByCW(GzCoord *coordList, GzCoord *normalList, GzRender *render, GzTextureIndex *uvList,
			  float *minX, float *minY, float *maxX, float *maxY){
	int minYIndex, minXIndex, maxXIndex, maxYIndex;
	bool minYminX, minYmaxX;
	*minX = FLT_MAX;
	*minY = FLT_MAX;
	*maxX = -FLT_MAX;
	*maxY = -FLT_MAX;

	for(int i = 0; i < 3; i++){
		if(coordList[i][0] < *minX){
			*minX = coordList[i][0];
			minXIndex = i;
		}
		if(coordList[i][0] > *maxX){
			*maxX = coordList[i][0];
			maxXIndex = i;
		}
		if(coordList[i][1] < *minY){
			*minY = coordList[i][1];
			minYIndex = i;
		}
		if(coordList[i][1] > *maxY){
			*maxY = coordList[i][1];
			maxYIndex = i;
		}
	}

	minYminX = (minYIndex == minXIndex) ? true : false;
	minYmaxX = (minYIndex == maxXIndex) ? true : false;

	if(minYIndex != 0){
		swapCoordList(coordList, normalList, uvList, 0, minYIndex);
		if(maxYIndex == 0)
			maxYIndex = minYIndex;
		if(maxXIndex == 0)
			maxXIndex = minYIndex;
	}
	if(minYminX && maxYIndex == 1)
		swapCoordList(coordList, normalList, uvList, 1, 2);
	else if(minYmaxX && maxYIndex != 1)
		swapCoordList(coordList, normalList, uvList, 1, 2);
	else if(!minYmaxX && !minYminX && maxXIndex != 1)
		swapCoordList(coordList, normalList, uvList, 1, 2);

	if(--*minX < .0f) *minX = .0f;
	if(*minX > render->display->xres) *minX = (float)render->display->xres;
	if(--*minY < .0f) *minY = .0f;
	if(*minY > render->display->yres) *minY = (float)render->display->yres;
	if(++*maxX > render->display->xres) *maxX = (float)render->display->xres;
	if(*maxX < .0f) *maxX = .0f;
	if(++*maxY > render->display->yres) *maxY = (float)render->display->yres;
	if(*maxY < .0f) *maxY = .0f;
}

int GzRotXMat(float degree, GzMatrix mat)
{
// Create rotate matrix : rotate along x axis
// Pass back the matrix using mat value

	if(!mat)
		return GZ_FAILURE;

	for(int i = 0; i < 4; i++)
		for(int j = 0; j < 4; j++)
			mat[i][j] = (i == j) ? 1.0f : .0f;
	float cos = cosf(degree * PI / 180.0f);
	float sin = sinf(degree * PI / 180.0f);
	mat[1][1] = cos;
	mat[2][2] = cos;
	mat[2][1] = sin;
	mat[1][2] = -sin;

	return GZ_SUCCESS;
}


int GzRotYMat(float degree, GzMatrix mat)
{
// Create rotate matrix : rotate along y axis
// Pass back the matrix using mat value

	if(!mat)
		return GZ_FAILURE;

	for(int i = 0; i < 4; i++)
		for(int j = 0; j < 4; j++)
			mat[i][j] = (i == j) ? 1.0f : .0f;
	float cos = cosf(degree * PI / 180.0f);
	float sin = sinf(degree * PI / 180.0f);
	mat[0][0] = cos;
	mat[2][2] = cos;
	mat[2][0] = -sin;
	mat[0][2] = sin;

	return GZ_SUCCESS;
}


int GzRotZMat(float degree, GzMatrix mat)
{
// Create rotate matrix : rotate along z axis
// Pass back the matrix using mat value
	
	if(!mat)
		return GZ_FAILURE;

	for(int i = 0; i < 4; i++)
		for(int j = 0; j < 4; j++)
			mat[i][j] = (i == j) ? 1.0f : .0f;
	float cos = cosf(degree * PI / 180.0f);
	float sin = sinf(degree * PI / 180.0f);
	mat[0][0] = cos;
	mat[1][1] = cos;
	mat[1][0] = sin;
	mat[0][1] = -sin;
	return GZ_SUCCESS;
}


int GzTrxMat(GzCoord translate, GzMatrix mat)
{
// Create translation matrix
// Pass back the matrix using mat value
	if(!mat)
		return GZ_FAILURE;

	for(int i = 0; i < 4; i++)
		for(int j = 0; j < 4; j++)
			mat[i][j] = (i == j) ? 1.0f : .0f;
	for(int i = 0; i < 3; i++)
		mat[i][3] = translate[i];
	return GZ_SUCCESS;
}


int GzScaleMat(GzCoord scale, GzMatrix mat)
{
// Create scaling matrix
// Pass back the matrix using mat value
	if(!mat)
		return GZ_FAILURE;

	for(int i = 0; i < 4; i++)
		for(int j = 0; j < 4; j++)
			mat[i][j] = (i == j) ? 1.0f : .0f;
	for(int i = 0; i < 3; i++)
		mat[i][i] = scale[i];
	return GZ_SUCCESS;
}


//----------------------------------------------------------
// Begin main functions

int GzNewRender(GzRender **render, GzRenderClass renderClass, GzDisplay	*display)
{
/*  
- malloc a renderer struct -
- keep closed until all inits are done -
- setup Xsp and anything only done once -
- span interpolator needs pointer to display -
- check for legal class GZ_Z_BUFFER_RENDER -
- init default camera -
*/ 
	if(renderClass != GZ_Z_BUFFER_RENDER)
		return GZ_FAILURE;

	if(!(*render = (GzRender *)malloc(sizeof(GzRender))))
		return GZ_FAILURE;

	GzRender empty = {NULL};

	memcpy(*render, &empty, sizeof(GzRender));
	(*render)->renderClass = renderClass;
	(*render)->display = display;
	(*render)->camera.FOV = DEFAULT_FOV;
	(*render)->camera.lookat[X] = .0f;
	(*render)->camera.lookat[Y] = .0f;
	(*render)->camera.lookat[Z] = .0f;

	(*render)->camera.position[X] = DEFAULT_IM_X;
	(*render)->camera.position[Y] = DEFAULT_IM_Y;
	(*render)->camera.position[Z] = DEFAULT_IM_Z;

	(*render)->camera.worldup[X] = .0f;
	(*render)->camera.worldup[Y] = 1.0f;
	(*render)->camera.worldup[Z] = .0f;

	(*render)->matlevel = 0;
	for(int i = 0; i < 4; i++)
		for(int j = 0; j < 4; j++)
			(*render)->Xsp[i][j] = (i == j) ? 1.0f : .0f;
		
	(*render)->Xsp[0][0] = (*render)->display->xres/2.0f;
	(*render)->Xsp[0][3] = (*render)->display->xres/2.0f;
	(*render)->Xsp[1][1] = (*render)->display->yres/-2.0f;
	(*render)->Xsp[1][3] = (*render)->display->yres/2.0f;
	(*render)->Xsp[3][3] = 1.0f;
	(*render)->offsetX = 0.0f;
	(*render)->offsetY = 0.0f;
	return GZ_SUCCESS;
}


int GzFreeRender(GzRender *render)
{
/* 
-free all renderer resources
*/
	if(render)
		free(render);
	render = NULL;

	return GZ_SUCCESS;
}


int GzBeginRender(GzRender *render)
{
/*  
- set up for start of each frame - clear frame buffer -
- compute Xiw and projection xform Xpi from camera definition 
- init Ximage - put Xsp at base of stack, push on Xpi and Xiw 
- now stack contains Xsw and app can push model Xforms if it want to. 
*/ 
	if(!render)
		return GZ_FAILURE;
	render->matlevel = 0;

	//Xsp, Xpi
	render->Xsp[2][2] = (float)Z_MAX * tanf(render->camera.FOV/2.0f*PI/180.0f);
	for(int i = 0; i < 4; i++)
		for(int j = 0; j < 4; j++)
			render->camera.Xpi[i][j] = (i == j) ? 1.0f : .0f;
	render->camera.Xpi[3][2] = tanf(render->camera.FOV/2.0f*PI/180.0f);

	//Xiw
	GzCoord x, y, z, up0;
	float dist, dp[] = {.0f, .0f, .0f}, dotProd = .0f;
	for(int i = 0; i < 3; i++)
		z[i] = render->camera.lookat[i] - render->camera.position[i];
	dist = sqrt((z[0]*z[0]) + (z[1]*z[1]) + (z[2]*z[2]));
	for(int i = 0; i < 3; i++){
		z[i] /= dist;
		dotProd += render->camera.worldup[i]*z[i];
	}
	for(int i = 0; i < 3; i++)
		up0[i] = render->camera.worldup[i] - (dotProd*z[i]);
	dist = sqrt((up0[0]*up0[0]) + (up0[1]*up0[1]) + (up0[2]*up0[2]));
	for(int i = 0; i < 3; i++)
		y[i] = up0[i]/dist;
	for(int i = 0; i < 3; i++)
		x[i] = y[(i+1) % 3]*z[(i+2) % 3] - y[(i+2) % 3]*z[(i+1) % 3];
	for(int i = 0; i < 3; i++){
		render->camera.Xiw[0][i] = x[i];
		render->camera.Xiw[1][i] = y[i];
		render->camera.Xiw[2][i] = z[i];
		render->camera.Xiw[3][i] = .0f;
	}
	for(int i = 0; i < 3; i++){
		dp[0] += x[i]*render->camera.position[i];
		dp[1] += y[i]*render->camera.position[i];
		dp[2] += z[i]*render->camera.position[i];
	}
	for(int i = 0; i < 3; i++){
		render->camera.Xiw[i][3] = -dp[i];
	}
	render->camera.Xiw[3][3] = 1.0f;

	GzPushMatrix(render, render->Xsp);
	GzPushMatrix(render, render->camera.Xpi);
	GzPushMatrix(render, render->camera.Xiw);

	render->open = 1;
	return GZ_SUCCESS;
}

int GzPutCamera(GzRender *render, GzCamera *camera)
{
/*
- overwrite renderer camera structure with new camera definition -
*/
	if(!render || !camera)
		return GZ_FAILURE;
	render->camera = *camera;
	return GZ_SUCCESS;	
}

int GzPushMatrix(GzRender *render, GzMatrix	matrix)
{
/*
- push a matrix onto the Ximage stack -
- check for stack overflow -
*/
	if(!render || !matrix || render->matlevel == MATLEVELS)
		return GZ_FAILURE;
	
	bool isRotMat = true;
	bool isXiw = true;
	for(int i = 0; i < 4; i++)
		for(int j = 0; j < 4; j++)
			if(matrix[i][j] !=  render->camera.Xiw[i][j]){
				isXiw = false;
				i = 4;
				j = 4;
			}

	for(int i = 0; i < 3; i++){
		float tot1 = .0f, tot2 = .0f;
		for(int j = 0; j < 3; j++){
			tot1 += matrix[i][j]*matrix[i][j];
			tot2 += matrix[j][i]*matrix[j][i];
		}
		if(tot1 > 1.01f || tot2 > 1.01f || tot1 < .99f || tot2 < .99f){
			isRotMat = false;
			i = 3;
		}
	}

	if(render->matlevel == 0){
		memcpy(render->Ximage[render->matlevel], matrix, sizeof(GzMatrix));
		if(!isRotMat && !isXiw){
			for(int i = 0; i < 4; i++){
				for(int j = 0; j < 4; j++)
					matrix[i][j] = (i == j) ? 1.0f : .0f;
			}
		}
		else if(isXiw){
			for(int i = 0; i < 3; i++){
				matrix[i][3] = .0f;
				matrix[3][i] = .0f;
			}	
			matrix[3][3] = 1.0f;
		}

		memcpy(render->Xnorm[render->matlevel], matrix, sizeof(GzMatrix));
	}

	else{
		memcpy(render->Ximage[render->matlevel], render->Ximage[render->matlevel-1], sizeof(GzMatrix));
		multMat(&render->Ximage[render->matlevel], matrix);

		memcpy(render->Xnorm[render->matlevel], render->Xnorm[render->matlevel-1], sizeof(GzMatrix));
		if(isRotMat || isXiw){
			if(isXiw){
				for(int i = 0; i < 3; i++){
					matrix[i][3] = .0f;
					matrix[3][i] = .0f;
				}	
				matrix[3][3] = 1.0f;
			}
			multMat(&render->Xnorm[render->matlevel], matrix);
		}
	}

	render->matlevel++;

	return GZ_SUCCESS;
}

int GzPopMatrix(GzRender *render)
{
/*
- pop a matrix off the Ximage stack -
- check for stack underflow -
*/
	if(!render || render->matlevel == 0)
		return GZ_FAILURE;
	render->matlevel--;
	memset(render->Ximage[render->matlevel], 0, sizeof(GzMatrix));
	return GZ_SUCCESS;
}


int GzPutAttribute(GzRender	*render, int numAttributes, GzToken	*nameList, 
	GzPointer	*valueList) /* void** valuelist */
{
/*
- set renderer attribute states (e.g.: GZ_RGB_COLOR default color)
- later set shaders, interpolaters, texture maps, and lights
*/
	if(!render || !nameList || !valueList || !render->open)
		return GZ_FAILURE;

	for(int i = 0; i < numAttributes; i++){
		switch(nameList[i]){
			case GZ_RGB_COLOR:
				memcpy(&(render->flatcolor), valueList[i], sizeof(GzColor));
				break;
			case GZ_INTERPOLATE:
				render->interp_mode = *(int*)valueList[i];
				break;
			case GZ_DIRECTIONAL_LIGHT:
				if(render->numlights < MAX_LIGHTS)
					memcpy(&(render->lights[render->numlights++]), valueList[i], sizeof(GzLight));
				else
					return GZ_FAILURE;
				break;
			case GZ_AMBIENT_LIGHT:
				memcpy(&(render->ambientlight), valueList[i], sizeof(GzLight));
				break;
			case GZ_AMBIENT_COEFFICIENT:
				memcpy(&(render->Ka), valueList[i], sizeof(GzColor));
				break;
			case GZ_DIFFUSE_COEFFICIENT:
				memcpy(&(render->Kd), valueList[i], sizeof(GzColor));
				break;
			case GZ_SPECULAR_COEFFICIENT:
				memcpy(&(render->Ks), valueList[i], sizeof(GzColor));
				break;
			case GZ_DISTRIBUTION_COEFFICIENT:
				render->spec = *(float*)valueList[i];
				break;
			case GZ_TEXTURE_MAP:
				render->tex_fun = (GzTexture)valueList[i];
				break;
			case GZ_AASHIFTX:
				render->offsetX = *(float*)valueList[i];
				break;
			case GZ_AASHIFTY:
				render->offsetY = *(float*)valueList[i];
				break;
			default:
				return GZ_FAILURE;
		}
	}	
	return GZ_SUCCESS;
}

int GzPutTriangle(GzRender	*render, int numParts, GzToken *nameList, 
				  GzPointer	*valueList)
/* numParts : how many names and values */
{
/*  
- pass in a triangle description with tokens and values corresponding to 
      GZ_POSITION:3 vert positions in model space 
- Xform positions of verts  -
- Clip - just discard any triangle with verts behind view plane 
       - test for triangles with all three verts off-screen 
- invoke triangle rasterizer  
*/ 
	if(!render || !nameList || !valueList || !render->open)
		return GZ_FAILURE;
	
	GzCoord coordList[3], normalList[3], triNormal;
	GzTextureIndex uvList[3], uvInterp;

	float minX, minY, maxX, maxY, vz[3], us[3], vs[3], temp[3];
	bool isInside = false;

	for(int i = 0; i < numParts; i++)
		switch (nameList[i]){
			case GZ_NULL_TOKEN:
				break;
			case GZ_POSITION:
				memcpy(&coordList, valueList[i], 3*sizeof(GzCoord));
				break;
			case GZ_NORMAL:
				memcpy(&normalList, valueList[i], 3*sizeof(GzCoord));
				break;
			case GZ_TEXTURE_INDEX:
				memcpy(&uvList, valueList[i], 3*sizeof(GzTextureIndex));
				break;
			default:
				return GZ_FAILURE;
		}

	//mult verts by Ximage, use AA offset, sort verts
	for(int k = 0; k < 3; k++){
		multMatCoord(render->Ximage[render->matlevel-1], coordList[k]);
		coordList[k][0] += render->offsetX;
		coordList[k][1] += render->offsetY;
	}
	sortByCW(coordList, normalList, render, uvList, &minX, &minY, &maxX, &maxY);

	//compute flat shading
	for(int i = 0; i < 3; i++)
		triNormal[i] = .33f*(normalList[0][i] + normalList[1][i] + normalList[2][i]);
	normalize(triNormal);
	computeColor(render->flatcolor, render, triNormal);

	//if coord is inside the frame, draw it
	for(int i = 0; i < 3; i++)
		if(coordList[i][0] > .0f && coordList[i][0] < render->display->xres &&
			coordList[i][1] > .0f && coordList[i][1] < render->display->yres &&
			coordList[i][2] > .0f){
			GzColor vertColors[3];

			//mult norms with Xnorm, find colors of verts, do persp. correction for uv coords
			for(int j = 0; j < 3; j++){
				multMatCoord(render->Xnorm[render->matlevel-1], normalList[j]);
				computeColor(vertColors[j], render, normalList[j]);

				vz[j] = coordList[j][2] / (Z_MAX - coordList[j][2]);
				us[j] = uvList[j][U] / vz[j];
				vs[j] = uvList[j][V] / vz[j];
			}


			for(int j = (int)minY*render->display->xres + (int)minX; 
				j < (int)maxY*render->display->xres + (int)maxX; j++){
						float x = (float)(j % render->display->xres);
						float y = (float)(j / render->display->xres);
						float z = interpolate(coordList, x, y);
					if(lee(coordList, render, x, y)){
						for(int k = 0; k < 3; k++){
							temp[k] = coordList[k][2];
							coordList[k][2] = us[k];
						}
						uvInterp[U] = interpolate(coordList, x, y);
						for(int k = 0; k < 3; k++)
							coordList[k][2] = vs[k];
						uvInterp[V] = interpolate(coordList, x, y);
						for(int k = 0; k < 3; k++)
							coordList[k][2] = temp[k];

						uvInterp[U] *= z/(Z_MAX-z+1);
						uvInterp[V] *= z/(Z_MAX-z+1);
						if(render->tex_fun(uvInterp[U], uvInterp[V], render->Ka) == GZ_FAILURE ||
							render->tex_fun(uvInterp[U], uvInterp[V], render->Kd) == GZ_FAILURE)
							return GZ_FAILURE;
						if(z <= render->display->fbuf[j].z && z > .0f)
							colorPixel(coordList, normalList, vertColors, render, x, y, z);
					}
				
					if(j % render->display->xres > maxX)
						j += render->display->xres - (int)maxX + (int)minX;
			}
			i = 3;
		}
	return GZ_SUCCESS;
}