/* Texture functions for cs580 GzLib	*/
#include    "stdafx.h" 
#include	"stdio.h"
#include	"Gz.h"

GzColor	*image;
int xs, ys;
int reset = 1;

/* Image texture function */
int tex_fun(float u, float v, GzColor color)
{
  unsigned char		pixel[3];
  unsigned char     dummy;
  char  		foo[8];
  int   		i, j;
  FILE			*fd;
  int a, b, c, d;
  float s, t;

  if (reset) {          /* open and load texture file */
    fd = fopen ("texture", "rb");
    if (fd == NULL) {
      fprintf (stderr, "texture file not found\n");
      exit(-1);
    }
    fscanf (fd, "%s %d %d %c", foo, &xs, &ys, &dummy);
    image = (GzColor*)malloc(sizeof(GzColor)*(xs+1)*(ys+1));
    if (image == NULL) {
      fprintf (stderr, "malloc for texture image failed\n");
      exit(-1);
    }

    for (i = 0; i < xs*ys; i++) {	/* create array of GzColor values */
      fread(pixel, sizeof(pixel), 1, fd);
      image[i][RED] = (float)((int)pixel[RED]) * (1.0 / 255.0);
      image[i][GREEN] = (float)((int)pixel[GREEN]) * (1.0 / 255.0);
      image[i][BLUE] = (float)((int)pixel[BLUE]) * (1.0 / 255.0);
      }

    reset = 0;          /* init is done */
	fclose(fd);
  }

	/* bounds-test u,v to make sure nothing will overflow image array bounds */
	if(u > 1.0f)
		u = 1.0f;
	if(u < .0f)
		u = .0f;
	if(v > 1.0f)
		v = 1.0f;
	if(v < .0f)
		v = .0f;

	u *= (float)(xs-1);
	v *= (float)(ys-1);

	/* determine texture cell corner values and perform bilinear interpolation */
	a = (int)u + (int)v*xs;
	b = (int)u+1 + (int)v*xs;
	c = (int)u+1 + ((int)v+1)*xs;
	d = (int)u + ((int)v+1)*xs;
	s = u - (float)(int)u;
	t = v - (float)(int)v;

	/* set color to interpolated GzColor value and return */
	for(int i = 0; i < 3; i++)
		color[i] = s*t*image[c][i] + (1-s)*t*image[d][i] + s*(1-t)*image[b][i] + (1-s)*(1-t)*image[a][i];
	return GZ_SUCCESS;
}


/* Procedural texture function */
int ptex_fun(float u, float v, GzColor color)
{
	u *= 255.0f;
	v *= 255.0f;
	int fourSquareSize = 64;
	if(((int)u%fourSquareSize < fourSquareSize/2 && (int)v%fourSquareSize < fourSquareSize/2) ||
		((int)u%fourSquareSize > fourSquareSize/2 && (int)v%fourSquareSize > fourSquareSize/2))
		for(int i = 0; i < 3; i++)
			color[i] = 0.0f;
	else
		for(int i = 0; i < 3; i++)
			color[i] = 1.0f;
	return GZ_SUCCESS;
}